use iced::{
    Alignment::Center,
    Element,
    widget::{self, Column, column, combo_box, text},
};
use std::{env, fmt::Display, fs, path::PathBuf};
use texts::*;

mod texts;

pub const BIBLE_DIRECTORY: &str = "./Holy-Bible-XML-Format";

fn main() -> iced::Result {
    let arg = env::args().collect::<Vec<String>>();
    if let Some(query) = arg.get(1) {
        if query.starts_with("H") | query.starts_with("h") {
            if let Some(found) = strong::get(query) {
                print!("{found}");
                return Ok(());
            }
        }
        if let Some(second_query) = arg.get(2) {
            bible::get(
                query,
                second_query,
                vec![format!("{BIBLE_DIRECTORY}/EnglishNASBBible.xml").into()],
            );
        }
    }

    iced::run("A cool counter", State::update, State::view)
}

#[derive(Debug, Clone)]
enum Message {
    BookSelected(&'static str),
    BibleSelected(String),
}

struct State {
    book: combo_box::State<&'static str>,
    selected: Option<&'static str>,
    files: combo_box::State<String>,
    file_selected: Option<String>,
}
impl Default for State {
    fn default() -> Self {
        let files = if let Ok(contents) = fs::read_dir(BIBLE_DIRECTORY) {
            let paths = contents
                .into_iter()
                .filter_map(|f| f.ok())
                .map(|f| (f.path(), f.file_type()))
                .filter(|(_, f)| f.is_ok())
                .map(|(a, b)| (a, b.unwrap()))
                .filter(|(_, f)| f.is_file())
                .map(|(f, _)| f)
                .filter(|f| f.extension().is_some())
                .filter(|f| f.extension().unwrap() == "xml")
                .collect::<Vec<PathBuf>>();
            paths
        } else {
            todo!()
        };
        let files = files
            .iter()
            .filter_map(|f| f.to_str())
            .map(|f| f.to_owned())
            .collect::<Vec<String>>();

        Self {
            files: combo_box::State::new(files),
            book: combo_box::State::new(texts::bible::BOOKS_IN_ORDER.to_vec()),
            selected: None,
            file_selected: None,
        }
    }
}

impl State {
    fn update(&mut self, msg: Message) {
        match msg {
            Message::BibleSelected(file) => self.file_selected = Some(file),
            Message::BookSelected(version) => self.selected = Some(version),
        }
    }

    fn view(&self) -> Element<Message> {
        column![
            widget::combo_box(
                &self.files,
                "Select Bible (type)",
                self.file_selected.as_ref(),
                Message::BibleSelected
            ),
            widget::combo_box(
                &self.book,
                "Select Book",
                self.selected.as_ref(),
                Message::BookSelected,
            ),

        ]
        .into()
    }
}
